# -*- coding: utf-8 -*-
# @Time    : 2019-04-16 11:56
# @Author  : WuTong
# @Email   : twu@aibee.com
# @FileName: config.py
# @Software: PyCharm

# DAG_FOLDER = '/user/local/airflow/dags'
DAG_FOLDER = 'static'
